﻿// /*
//  * Author: dtobi
//  * This work is shared under CC BY-NC-ND 3.0 license.
//  * Non commercial, no derivatives, attribution if shared unmodified.
//  * You may distribute this code and the compiled .dll as is.
//  * 
//  * Exception from the no-deriviates clause in case of KSP updates:
//  * In case of an update of KSP that breaks the code, you may change
//  * this code to make it work again and redistribute it under a different
//  * class name until the author publishes an updated version. After a
//  * release by the author, the right to redistribute the changed code
//  * vanishes.
//  * 
//  * You must keep this boilerplate in the file and give credit to the author
//  * in the download file as well as on the webiste that links to the file.
//  * 
//  * Should you wish to change things in the code, contact me via the KSP forum.
//  * Patches are welcome.
//  *
//  */
//
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using UnityEngine;



namespace KM_Lib
{

    public class km_WetWorkshop : PartModule
    {
        const int kerbalSpaceCost = 130;
        const int kerbalSupplyCost = 50;

        [KSPField (isPersistant = true)]
        private int crewCapacity = 0;

        private bool createLivingspace (Part p)
        {
            if (this.part.Resources ["Supplies"] == null || this.part.Resources ["EmptySpace"] == null) {
                print ("KM Wet workshops: Resources not available");
                return false;
            }
            if (this.part.Resources ["Supplies"].amount >= kerbalSupplyCost && this.part.Resources ["EmptySpace"].amount >= kerbalSpaceCost) {
                this.part.CrewCapacity++;
                crewCapacity++;
                this.part.Resources ["EmptySpace"].amount -= kerbalSpaceCost;
                this.part.Resources ["Supplies"].amount -= kerbalSupplyCost;

                p.mass += PartResourceLibrary.Instance.GetDefinition ("Supplies").density * kerbalSupplyCost;
                return true;
            }
            return false;
        }

        private bool convertTank (Part p)
        {

            if (p != null) {
                print ("Converting tank:" + p.name);


                foreach (PartResource resource in p.Resources) {
                    if (resource != null) {
                        print ("Check:" + resource.resourceName);
                        if (resource.amount > 0.2) {
                            print (resource.resourceName + " is not empty.");
                            return false;
                        }
                    }
                }
                foreach (PartResource resource in p.Resources) {
                    if (resource != null && resource.resourceName != "EmptySpace") {
                        print (resource.resourceName + "  removed");

                        this.part.Resources ["EmptySpace"].maxAmount += resource.maxAmount / PartResourceLibrary.Instance.GetDefinition (resource.resourceName).density / 1000;
                        this.part.Resources ["EmptySpace"].amount += resource.maxAmount / PartResourceLibrary.Instance.GetDefinition (resource.resourceName).density / 1000;
                        resource.maxAmount = 0;
                        resource.amount = 0;
                        this.part.Resources ["Supplies"].maxAmount = 300;
                        print ("Mass after:" + p.mass);
                    }
                }
                this.part.CrewCapacity = 1;
            }
            return true;
        }

        [KSPAction ("Convert Space")]
        public void toggletAG (KSPActionParam param)
        {

            convertTank (this.part.parent);
        }

        [KSPEvent (guiName = "Convert Space", guiActive = true, guiActiveEditor = false)]
        public void ConvertSpaceAG ()
        {
            convertTank (this.part.parent);

        }

        [KSPEvent (guiName = "Create Livingspace", guiActive = true, guiActiveEditor = false)]
        public void ConvertSpace ()
        {
            createLivingspace (this.part.parent);
        }


        public override void OnStart (StartState state)
        {

            this.part.CrewCapacity = crewCapacity;
            this.part.parent.CrewCapacity = (crewCapacity > 0 ? 1 : 0);
            base.OnStart (state);


           
           

        }

    }



}

